@@ -1,80 +1,72 @@
 """
-GUI framework demo implementations
-==================================
-
-This module is used as showcase base for the implementation of demo
-applications of the GUI framework packages provided
-by the ae namespace.
-
-The plan is to integrate the following GUI frameworks on
-top of the :class:`abstract base class <~ae.gui_app.MainAppBase>`
-(implemented in the :mod:`ae.gui_app` portion of the
-ae namespace):
-
-* :mod:`Kivy <ae.kivy_app>` based on the `kivy framework <https://kivy.org>`__:
-  `kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>`__
-* :mod:`Enaml <ae.enaml_app>` based on `the enaml framework <https://enaml.readthedocs.io/en/latest/>`__:
-  `enaml lisz demo app <https://gitlab.com/ae-group/enaml_lisz>`__
-* :mod:`Beeware Toga <ae.toga_app>` based on `the beeware framework <https://beeware.org>`__:
-  `beeware toga lisz demo app <https://gitlab.com/ae-group/toga_lisz>`__
-* :mod:`Dabo <ae.dabo_app>` based on `the dabo framework <https://dabodev.com/>`__:
-  `dabo lisz demo app <https://gitlab.com/ae-group/dabo_lisz>`__
-* :mod:`pyglet <ae.pyglet_app>`
-* :mod:`pygobject <ae.pygobject_app>`
-* :mod:`AppJar <ae.appjar_app>`
-
-The main app base mixin class :class:`LiszDataMixin` provided by this module
-is used for to manage the common data structures, functions and methods
-for the various demo applications variants based on :mod:`ae.gui_app`
-and the related GUI framework implementation portions
-(like e.g. :mod:`ae.kivy_app` and :mod:`ae.enaml_app`) of the ae namespace.
-
-Further down in this module you find additional notes on each
-of the already implemented GUI frameworks.
-
-
 Lisz Demo App
 =============
 
-The Lisz demo app is managing a recursive item tree that can be used
-e.g. as to-do or shopping list.
+This module provides common constants, functions and methods for the showcase/demo application `Lisz`, which is
+demonstrating the usage of the GUI framework packages provided by the `ae namespace <https://ae.readthedocs.io>`__.
+
+
+usage demonstration of ae namespace portions
+--------------------------------------------
+
+The usage of the following ae namespace portions get demonstrated by this application:
+
+* :mod:`ae.base`: basic constants and helper functions
+* :mod:`ae.files`: file collection, grouping and caching
+* :mod:`ae.deep`: deep data structure search and replace
+* :mod:`ae.i18n`: internationalization / Localization helpers
+* :mod:`ae.paths`: generic file path helpers
+* :mod:`ae.inspector`: inspection and debugging helper functions
+* :mod:`ae.updater`: application environment updater
+* :mod:`ae.core`: application core constants, helper functions and base classes
+* :mod:`ae.literal`: literal type detection and evaluation
+* :mod:`ae.console`: console application environment
+* :mod:`ae.parse_date`: parse date strings more flexible and less strict
+* :mod:`ae.gui_app`: base class for python applications with a graphical user interface
+* :mod:`ae.gui_help`: main app base class with context help for flow and app state changes
+
+
+.. hint::
+    The Kivy variant of this demo app is additionally using the following ae namespace portions:
+    :mod:`ae.kivy_auto_width`, :mod:`ae.kivy_dyn_chi`, :mod:`ae.kivy_help`, :mod:`ae.kivy_relief_canvas`,
+    :mod:`ae.kivy_app` and :mod:`ae.kivy_user_prefs`.
 
 
 features of the Lisz demo app
 -----------------------------
 
-* recursive item data tree manipulation: add, edit and delete item
-* each item can be selected/check marked
-* easy filtering of selected/checked and unselected/unchecked items
-* each item can be converted to a sub-node (sub-list) and back to a leaf item
-* item order changeable via drag & drop
-* item movable into parent and sub node
-* easy navigation within the item tree (up/down navigation in tree and quick jump)
-* simple demo help screen
-* colors changeable by user
-* font and button sizes are changeable by user
+* internationalization of texts, user messages, help texts, button/label texts (:mod:`ae.i18n`)
+* easy mapping of files in complex folder structures (:mod:`ae.files`, :mod:`ae.paths`)
+* providing help screens (:mod:`ae.gui_help`, :mod:`ae.kivy_help`)
+* colors changeable by user (:mod:`ae.kivy_user_prefs`, :mod:`ae.enaml_app`)
+* font and button sizes are changeable by user (:mod:`ae.gui_app`)
 * dark and light theme switchable by user
 * sound output support with sound volume configurable by user
-* vibration output support with amplitude configurable by user (only for mobile apps)
+* recursive item data tree manipulation: add, edit and delete item
+* each item can be selected/check marked
+* filtering of selected/checked and unselected/unchecked items
+* an item represents either a sub-node (sub-list) or a leaf of the data tree
+* item order changeable via drag & drop
+* item can be moved to the parent or a sub-node
+* easy navigation within the item tree (up/down navigation in tree and quick jump)
 
 
 lisz application data model
 ---------------------------
 
-To keep it simply, the data managed by the Lisz application is
-a minimalistic tree structure that gets stored as
-an :ref:`application status`. The root node and with that
-the whole recursive data structure
-gets stored in the app state variable `root_node`,
-without the need of any database.
+The Lisz demo app is managing a recursive item tree that can be used e.g. as to-do or
+shopping list.
 
-The root of the tree structure is a list of the type `LiszNode`
-containing list items of type `LiszItem`. A `LiszItem` element
-represents a dict of the type `Dict[str, Any]`.
+To keep this demo app simple, the data managed by the Lisz application is a minimalistic
+tree structure that gets stored as an :ref:`application status`, without the need of any
+database. The root node and with that the whole recursive data structure gets stored in
+the app state variable `root_node`.
 
-Each `LiszItem` element of the tree structure is either a
-leaf or a node. And each node is a sub-list with a recursive
-structure identical to the root node and of the type `LiszNode`.
+The root of the tree structure is a list of the type `LiszNode` containing list items of
+type `LiszItem`. A `LiszItem` element represents a dict of the type `Dict[str, Any]`.
+
+Each `LiszItem` element of the tree structure is either a leaf or a node. And each node is a
+sub-list with a recursive structure identical to the root node and of the type `LiszNode`.
 
 The following graph is showing an example data tree:
 
@@ -93,15 +85,39 @@ The following graph is showing an example data tree:
         rec3:CBA -> "Leaf Item CBA"
     }
 
-The above example tree structure is containing
-the root node items `A` -which is a leaf - and `C` - which is
-a sub-node.
+The above example tree structure is containing the root node items `A` (which is a leaf)
+and `C` (which is a sub-node).
 
-The node `C` consists of the items `CA` and `CB` where `CA` is
-a leaf and `CB` is a node.
+The node `C` consists of the items `CA` and `CB` where `CA` is a leaf and `CB` is a node.
 
-Finally the first item of the node `CB` is another
-sub-node with the leaf item `CBA`.
+Finally the first item of the node `CB` is another sub-node with the leaf item `CBA`.
+
+
+GUI framework demo implementations
+==================================
+
+The plan is to integrate the following GUI frameworks on
+top of the :class:`abstract base class <~ae.gui_app.MainAppBase>`
+(implemented in the :mod:`ae.gui_app` portion of the
+`ae namespace <https://ae.readthedocs.io`__):
+
+* :mod:`Kivy <ae.kivy_app>` based on the `kivy framework <https://kivy.org>`__:
+  `kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>`__
+* :mod:`Enaml <ae.enaml_app>` based on `the enaml framework <https://enaml.readthedocs.io/en/latest/>`__:
+  `enaml lisz demo app <https://gitlab.com/ae-group/enaml_lisz>`__
+* :mod:`Beeware Toga <ae.toga_app>` based on `the beeware framework <https://beeware.org>`__:
+  `beeware toga lisz demo app <https://gitlab.com/ae-group/toga_lisz>`__
+* :mod:`Dabo <ae.dabo_app>` based on `the dabo framework <https://dabodev.com/>`__:
+  `dabo lisz demo app <https://gitlab.com/ae-group/dabo_lisz>`__
+* :mod:`pyglet <ae.pyglet_app>`
+* :mod:`pygobject <ae.pygobject_app>`
+* :mod:`AppJar <ae.appjar_app>`
+
+The main app base mixin class :class:`LiszDataMixin` provided by this module
+is used for to manage the common data structures, functions and methods
+for the various demo applications variants based on :mod:`ae.gui_app`
+and the related GUI framework implementation portions
+(like e.g. :mod:`ae.kivy_app` and :mod:`ae.enaml_app`) of the ae namespace.
 
 
 gui framework implementation variants
@@ -170,7 +186,7 @@ from ae.i18n import get_f_string
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action   # type: ignore
 
 
-__version__ = '0.1.35'
+__version__ = '0.1.36'
 
 
 FLOW_PATH_ROOT_ID = 'ROOT'  #: pseudo item id needed for flow path jumper and for drop onto leave item button
@@ -415,7 +431,7 @@ class LiszDataMixin(ABC):
             item.pop('sel', None)       # always first remove sel left-overs
             if 'node' in item:
                 self.change_sub_node_sel(item['node'], set_sel_to)
-            elif set_sel_to:
+            if set_sel_to:
                 item['sel'] = 1.0
 
     def current_item_or_node_literal(self) -> str:
@@ -434,18 +450,27 @@ class LiszDataMixin(ABC):
                 lit = item['id']
         return lit
 
-    def delete_item(self, item_id: str, node_only: bool = False):
-        """ delete either the complete item or the sub node of the item (identified by the passed item id).
+    def delete_items(self, *item_ids: str, parent_node: Optional[LiszNode] = None, node_only: bool = False) -> LiszNode:
+        """ delete either complete items or sub node of the items (identified by the passed item ids).
 
-        :param item_id:         item id to identify the item/sub-node to be deleted.
+        :param item_ids:        tuple of item ids to identify the items/sub-nodes to be deleted.
+        :param parent_node:     node from where the item has to be removed from (default=current node).
         :param node_only:       True if only delete the sub-node of the identified item, False for to delete the item.
         """
-        nid = self.item_by_id(item_id)
-        if node_only:
-            nid.pop('node')
-        else:
-            assert nid in self.current_node_items, f"DEL item data: {nid} not in {self.current_node_items}"
-            self.current_node_items.remove(nid)
+        if parent_node is None:
+            parent_node = self.current_node_items
+
+        del_items = list()
+        for item_id in item_ids:
+            nid = self.item_by_id(item_id, searched_node=parent_node)
+            if node_only:
+                del_items.extend(nid.pop('node'))
+            else:
+                assert nid in parent_node, f"DEL item data: {nid} not in {parent_node}"
+                parent_node.remove(nid)
+                del_items.append(nid)
+
+        return del_items
 
     def edit_validate(self, old_item_index: int, new_id: Optional[str] = None, want_node: Optional[bool] = None,
                       parent_node: Optional[LiszNode] = None, new_item_index: int = 0) -> str:
@@ -891,67 +916,6 @@ class LiszDataMixin(ABC):
 
         return True
 
-    def on_key_press(self, modifiers: str, key_code: str) -> bool:
-        """  check key press event for to be handled and processed as command/action.
-
-        :param modifiers:       modifier keys string.
-        :param key_code:        key code string.
-        :return:                True if key event got processed/used, else False.
-        """
-        if modifiers == 'Ctrl' and key_code in ('c', 'v', 'x'):
-            if self.call_method('on_clipboard_key_' + key_code):    # redirect to framework-specific implementation
-                return True
-
-        elif key_code == 'r':
-            if modifiers == 'Shift':
-                self.change_app_state('flow_path', [])  # quick jump to root (also repairing any flow path errors)
-                self.change_app_state('flow_id', '')
-            self.refresh_all()
-            return True
-
-        if modifiers or flow_action(self.flow_id) not in ('', 'focus'):
-            return False
-
-        # handle hot key without a modifier key and while in item list, first current item flow changes
-        flo_key = flow_key(self.flow_id)
-        if key_code == 'up':
-            self.focus_neighbour_item(-1)
-        elif key_code == 'down':
-            self.focus_neighbour_item(1)
-        elif key_code == 'pgup':
-            self.focus_neighbour_item(-15)
-        elif key_code == 'pgdown':
-            self.focus_neighbour_item(15)
-        elif key_code == 'home':
-            self.focus_neighbour_item(-999999)
-        elif key_code == 'end':
-            self.focus_neighbour_item(999999)
-
-        # toggle selection of current item
-        elif key_code == ' ' and flo_key:    # key string 'space' is not in Window.command_keys
-            self.change_flow(id_of_flow('toggle', 'item_sel', flo_key))
-
-        # enter/leave flow in current list
-        elif key_code in ('enter', 'right') and flo_key and 'node' in self.item_by_id(flo_key):
-            self.change_flow(id_of_flow('enter', 'item', flo_key))
-        elif key_code in ('escape', 'left') and self.flow_path:
-            self.change_flow(id_of_flow('leave', 'item'))
-
-        # item processing: add, edit or request confirmation of deletion of current item
-        elif key_code in ('a', '+'):
-            self.change_flow(id_of_flow('add', 'item'))
-
-        elif key_code == 'e' and flo_key:
-            self.change_flow(replace_flow_action(self.flow_id, 'edit'))  # popup_kwargs=dict(parent=self.framework_win))
-
-        elif key_code in ('-', 'del') and flo_key:
-            self.change_flow(id_of_flow('confirm', 'item_deletion', flo_key))
-
-        else:
-            return False    # pressed key not processable in the current flow/app-state
-
-        return True         # key press processed
-
     def on_item_enter(self, _key: str, event_kwargs: dict) -> bool:
         """ entering sub node from current node.
 
@@ -991,7 +955,7 @@ class LiszDataMixin(ABC):
         :param item_id:         item id of the leaf/node to toggle selection for.
         :param event_kwargs:    event kwargs, containing a `set_sel_to` key with a boolean value, where
                                 True will select and False deselect the item (or the sub-items if the item is a
-                                non-empty sub-node).
+                                non-empty node).
         :return:                True for to process/change flow id.
 
         This flow change event can be used alternatively to :meth:`~LiszDataMixin.on_item_sel_toggle`
@@ -1000,15 +964,80 @@ class LiszDataMixin(ABC):
         """
         node_idx = self.find_item_index(item_id)
         set_sel_to = event_kwargs['set_sel_to']
-        node = self.current_node_items[node_idx].get('node', list())
-        if node:
+        item = self.current_node_items[node_idx]
+        node = item.get('node')
+        if node is not None:
             self.change_sub_node_sel(node, set_sel_to)
+            if set_sel_to:
+                item['sel'] = 1.0
         else:
-            # assert not set_sel_to == bool(self.current_node_items[node_idx].get('sel'))
             self.toggle_item_sel(node_idx)
+        event_kwargs['changed_event_name'] = 'refresh_node_widgets'
         event_kwargs['flow_id'] = id_of_flow('focus', 'item', item_id)
         return True
 
+    def on_key_press(self, modifiers: str, key_code: str) -> bool:
+        """  check key press event for to be handled and processed as command/action.
+
+        :param modifiers:       modifier keys string.
+        :param key_code:        key code string.
+        :return:                True if key event got processed/used, else False.
+        """
+        if modifiers == 'Ctrl' and key_code in ('c', 'v', 'x'):
+            if self.call_method('on_clipboard_key_' + key_code):    # redirect to framework-specific implementation
+                return True
+
+        elif key_code == 'r':
+            if modifiers == 'Shift':
+                self.change_app_state('flow_path', [])  # quick jump to root (also repairing any flow path errors)
+                self.change_app_state('flow_id', '')
+            self.refresh_all()
+            return True
+
+        flo_act = flow_action(self.flow_id)
+        if modifiers or flo_act not in ('', 'focus'):
+            return False
+
+        # handle hot key without a modifier key and while in item list, first current item flow changes
+        focused_id = flo_act == 'focus' and flow_key(self.flow_id)
+        if key_code == 'up':
+            self.focus_neighbour_item(-1)
+        elif key_code == 'down':
+            self.focus_neighbour_item(1)
+        elif key_code == 'pgup':
+            self.focus_neighbour_item(-15)
+        elif key_code == 'pgdown':
+            self.focus_neighbour_item(15)
+        elif key_code == 'home':
+            self.focus_neighbour_item(-999999)
+        elif key_code == 'end':
+            self.focus_neighbour_item(999999)
+
+        # toggle selection of current item
+        elif key_code == ' ' and focused_id:    # key string 'space' is not in Window.command_keys
+            self.change_flow(id_of_flow('toggle', 'item_sel', focused_id))
+
+        # enter/leave flow in current list
+        elif key_code in ('enter', 'right') and focused_id and 'node' in self.item_by_id(focused_id):
+            self.change_flow(id_of_flow('enter', 'item', focused_id))
+        elif key_code in ('escape', 'left') and self.flow_path:
+            self.change_flow(id_of_flow('leave', 'item'))
+
+        # item processing: add, edit or request confirmation of deletion of current item
+        elif key_code in ('a', '+'):
+            self.change_flow(id_of_flow('add', 'item'))
+
+        elif key_code == 'e' and focused_id:
+            self.change_flow(replace_flow_action(self.flow_id, 'edit'))  # popup_kwargs=dict(parent=self.framework_win))
+
+        elif key_code in ('-', 'del') and focused_id:
+            self.change_flow(id_of_flow('confirm', 'item_deletion', focused_id))
+
+        else:
+            return False    # pressed key not processable in the current flow/app-state
+
+        return True         # key press processed
+
     def on_node_extract(self, flow_path_text: str, event_kwargs: dict) -> bool:
         """ extract the leaves of the node specified by `flow_path_text`.
 
@@ -1085,7 +1114,7 @@ class LiszDataMixin(ABC):
         flow_path = self.flow_path_from_text(flow_path_text)
 
         # cannot close popup here because the close event will be processed in the next event loop
-        # an because flow_path_from_text() is overwriting the open popup action in self.flow_path
+        # and because flow_path_from_text() is overwriting the open popup action in self.flow_path
         # we have to re-add the latest flow id entry from the current/old flow path that opened the jumper
         # here (for it can be removed by FlowPopup closed event handler when the jumper popup closes).
         # open_jumper_flow_id = id_of_flow('open', 'flow_path_jumper')
@@ -1160,39 +1189,37 @@ class LiszDataMixin(ABC):
         for item in del_items:
             node.remove(item)
 
-    def sub_item_ids(self, node: Optional[LiszNode] = None, item_id: str = '',
+    def sub_item_ids(self, node: Optional[LiszNode] = None, item_ids: Tuple[str, ...] = (),
                      leaves_only: bool = True, hide_sel_val: Optional[bool] = None, recursive: bool = True,
                      sub_ids: Optional[List[str]] = None) -> List[str]:
-        """ return item names of all items including their sub-node items (if exists and recursive==True).
+        """ return item names/ids of the specified items including their sub-node items (if exists and recursive==True).
 
-        Used for to determine the affected items if user want to delete or de-/select the sub-items of
-        the node item specified by the :paramref:`sub_item_ids.item_id` argument.
+        Used for to determine the affected item ids if user want to delete or de-/select the sub-items of
+        the item(s) specified by the passed arguments.
 
-        :param node:            searched node, if not passed use root node if the :paramref:`~sub_item_ids.item_id`
-                                argument is :data:`FLOW_PATH_ROOT_ID` else use the current node as default.
-        :param item_id:         item id of the node item to be investigated (for to be deleted or de-/selected later).
-                                Pass FLOW_PATH_ROOT_ID or '' for to get the item names/ids within AND underneath
-                                of the node specified by :paramref:`~sub_item_ids.node` (or its default value).
-        :param leaves_only:     True (the default) if only include the leaf item ids of the sub-nodes of the node item.
-        :param hide_sel_val:    pass False/True for to exclude un-/selected items from the returned list of ids.
+        :param node:            searched node, if not passed use the current node as default.
+        :param item_ids:        optional item id filter, if passed only items with an id in this tuple will be returned.
+                                This filter will not be used for sub-node filtering (if recursive==True).
+        :param leaves_only:     pass False to also include/return node item ids.
+        :param hide_sel_val:    pass False/True for to exclude un-/selected leaf items from the returned list of ids.
                                 If None or not passed then all found items will be included.
         :param recursive:       pass False if only the passed start node has to be investigated/included.
-        :param sub_ids:         already found sub item ids (used only for the recursive calls of this method).
+        :param sub_ids:         already found sub item ids (used for the recursive calls of this method).
         :return:                list of found item ids.
         """
         if node is None:
-            node = self.root_node if item_id == FLOW_PATH_ROOT_ID else self.current_node_items
-        if item_id not in ('', FLOW_PATH_ROOT_ID):
-            node = self.item_by_id(item_id, searched_node=node).get('node', list())
+            node = self.current_node_items
         if sub_ids is None:
             sub_ids = list()
 
         for item in node:           # type: ignore # mypy does not recognize that node cannot be None
+            if item_ids and item['id'] not in item_ids:
+                continue
             if (not leaves_only) if 'node' in item else (hide_sel_val is None or bool(item.get('sel')) != hide_sel_val):
                 sub_ids.append(item['id'])
-            if recursive and item.get('node'):
-                self.sub_item_ids(node=node, item_id=item['id'], leaves_only=leaves_only, hide_sel_val=hide_sel_val,
-                                  sub_ids=sub_ids)
+            sub_node = item.get('node')
+            if recursive and sub_node:
+                self.sub_item_ids(node=sub_node, leaves_only=leaves_only, hide_sel_val=hide_sel_val, sub_ids=sub_ids)
 
         return sub_ids
 
